@@ -1,23 +1,9 @@
-#
 # Copyright (C) 2006-2008, 2015 Red Hat, Inc.
 # Copyright (C) 2006 Daniel P. Berrange <berrange@redhat.com>
 # Copyright (C) 2010 Marc-Andre Lureau <marcandre.lureau@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import logging
 
@@ -232,11 +218,13 @@ class vmmConsolePages(vmmGObjectUI):
         self._overlay_toolbar = Gtk.Toolbar()
         self._overlay_toolbar.set_show_arrow(False)
         self._overlay_toolbar.set_style(Gtk.ToolbarStyle.BOTH_HORIZ)
+        self._overlay_toolbar.get_accessible().set_name("Fullscreen Toolbar")
 
         # Exit fullscreen button
         button = Gtk.ToolButton.new_from_stock(Gtk.STOCK_LEAVE_FULLSCREEN)
         button.set_tooltip_text(_("Leave fullscreen"))
         button.show()
+        button.get_accessible().set_name("Fullscreen Exit")
         self._overlay_toolbar.add(button)
         button.connect("clicked", self._leave_fullscreen)
 
@@ -265,6 +253,7 @@ class vmmConsolePages(vmmGObjectUI):
         self._send_key_button.set_tooltip_text(_("Send key combination"))
         self._send_key_button.show_all()
         self._send_key_button.connect("clicked", keycombo_menu_clicked)
+        self._send_key_button.get_accessible().set_name("Fullscreen Send Key")
         self._overlay_toolbar.add(self._send_key_button)
 
         self._timed_revealer = _TimedRevealer(self._overlay_toolbar)
@@ -399,12 +388,12 @@ class vmmConsolePages(vmmGObjectUI):
         if align_ratio > desktop_ratio:
             desktop_w = int(req.height * desktop_ratio)
             desktop_h = req.height
-            dx = (req.width - desktop_w) / 2
+            dx = (req.width - desktop_w) // 2
 
         else:
             desktop_w = req.width
-            desktop_h = int(req.width / desktop_ratio)
-            dy = (req.height - desktop_h) / 2
+            desktop_h = int(req.width // desktop_ratio)
+            dy = (req.height - desktop_h) // 2
 
         viewer_alloc = Gdk.Rectangle()
         viewer_alloc.x = dx
@@ -455,14 +444,14 @@ class vmmConsolePages(vmmGObjectUI):
         if not self._viewer.console_get_desktop_resolution():
             return
 
-        topwin_alloc = self.topwin.get_allocation()
+        top_w, top_h = self.topwin.get_size()
         viewer_alloc = self.widget("console-gfx-scroll").get_allocation()
         desktop_w, desktop_h = self._viewer.console_get_desktop_resolution()
 
         self.topwin.unmaximize()
         self.topwin.resize(
-            desktop_w + (topwin_alloc.width - viewer_alloc.width),
-            desktop_h + (topwin_alloc.height - viewer_alloc.height))
+            desktop_w + (top_w - viewer_alloc.width),
+            desktop_h + (top_h - viewer_alloc.height))
 
 
     ################
@@ -609,7 +598,8 @@ class vmmConsolePages(vmmGObjectUI):
         self._close_viewer()
         self.widget("console-pages").set_current_page(
             _CONSOLE_PAGE_UNAVAILABLE)
-        self.widget("console-unavailable").set_label("<b>" + msg + "</b>")
+        if msg:
+            self.widget("console-unavailable").set_label("<b>" + msg + "</b>")
 
     def _activate_auth_page(self, withPassword, withUsername):
         (pw, username) = self.config.get_console_password(self.vm)
@@ -652,12 +642,18 @@ class vmmConsolePages(vmmGObjectUI):
             self.widget("console-pages").get_nth_page(i).set_visible(
                 i == newpage)
 
+        # Dispatch the next bit in idle_add, so the UI size can change
         self.idle_add(self._refresh_widget_states)
 
     def _refresh_widget_states(self):
+        if not self.vm:
+            # This is triggered via cleanup + idle_add, so vm might
+            # disappear and spam the logs
+            return
+
         pagenum = self.widget("console-pages").get_current_page()
         paused = self.vm.is_paused()
-        is_viewer = (pagenum == _CONSOLE_PAGE_VIEWER and
+        is_viewer = bool(pagenum == _CONSOLE_PAGE_VIEWER and
             self._viewer and self._viewer.console_is_open())
 
         self.widget("details-menu-vm-screenshot").set_sensitive(is_viewer)
@@ -685,11 +681,11 @@ class vmmConsolePages(vmmGObjectUI):
 
         ginfo = None
         try:
-            gdevs = self.vm.get_graphics_devices()
+            gdevs = self.vm.xmlobj.devices.graphics
             gdev = gdevs and gdevs[0] or None
             if gdev:
                 ginfo = ConnectionInfo(self.vm.conn, gdev)
-        except Exception, e:
+        except Exception as e:
             # We can fail here if VM is destroyed: xen is a bit racy
             # and can't handle domain lookups that soon after
             logging.exception("Getting graphics console failed: %s", str(e))
@@ -732,7 +728,7 @@ class vmmConsolePages(vmmGObjectUI):
             self._refresh_enable_accel()
 
             self._viewer.console_open()
-        except Exception, e:
+        except Exception as e:
             logging.exception("Error connection to graphical console")
             self._activate_unavailable_page(
                     _("Error connecting to graphical console") + ":\n%s" % e)
@@ -786,7 +782,6 @@ class vmmConsolePages(vmmGObjectUI):
             self._enable_modifiers()
 
     def _viewer_auth_rejected(self, ignore, errmsg):
-        self._close_viewer()
         self._activate_unavailable_page(errmsg)
 
     def _viewer_auth_error(self, ignore, errmsg, viewer_will_disconnect):
@@ -797,7 +792,6 @@ class vmmConsolePages(vmmGObjectUI):
             # GtkVNC will disconnect after an auth error, so lets do it for
             # them and re-init the viewer (which will be triggered by
             # update_vm_widget_states if needed)
-            self._close_viewer()
             self._activate_unavailable_page(errmsg)
 
         self._update_vm_widget_states()
@@ -829,9 +823,7 @@ class vmmConsolePages(vmmGObjectUI):
         self._activate_unavailable_page(msg)
 
     def _viewer_disconnected(self, ignore, errdetails, ssherr):
-        self.widget("console-pages").set_current_page(
-            _CONSOLE_PAGE_UNAVAILABLE)
-        self._close_viewer()
+        self._activate_unavailable_page(_("Viewer disconnected."))
         logging.debug("Viewer disconnected")
 
         # Make sure modifiers are set correctly
@@ -872,7 +864,8 @@ class vmmConsolePages(vmmGObjectUI):
         """
         Find the default graphical or serial console for the VM
         """
-        if self.vm.get_graphics_devices() or not self.vm.get_serial_devs():
+        if (self.vm.xmlobj.devices.graphics or
+            not self.vm.get_serialcon_devices()):
             return
 
         # We iterate through the 'console' menu and activate the first
@@ -890,11 +883,11 @@ class vmmConsolePages(vmmGObjectUI):
     def _console_menu_toggled(self, src, dev):
         self.widget("details-pages").set_current_page(DETAILS_PAGE_CONSOLE)
 
-        if dev.virtual_device_type == "graphics":
+        if dev.DEVICE_TYPE == "graphics":
             self.widget("console-pages").set_current_page(_CONSOLE_PAGE_VIEWER)
             return
 
-        target_port = dev.vmmindex
+        target_port = dev.get_xml_idx()
         serial = None
         name = src.get_label()
         for s in self._serial_consoles:
@@ -919,7 +912,7 @@ class vmmConsolePages(vmmGObjectUI):
         self.widget("serial-pages").set_current_page(page_idx)
 
     def _build_serial_menu_items(self, menu_item_cb):
-        devs = self.vm.get_serial_devs()
+        devs = self.vm.get_serialcon_devices()
         if len(devs) == 0:
             menu_item_cb(_("No text console available"),
                          radio=False, sensitive=False)
@@ -933,10 +926,10 @@ class vmmConsolePages(vmmGObjectUI):
                 active_label = self._serial_consoles[serial_page].name
 
         for dev in devs:
-            if dev.virtual_device_type == "console":
-                label = _("Text Console %d") % (dev.vmmindex + 1)
+            if dev.DEVICE_TYPE == "console":
+                label = _("Text Console %d") % (dev.get_xml_idx() + 1)
             else:
-                label = _("Serial %d") % (dev.vmmindex + 1)
+                label = _("Serial %d") % (dev.get_xml_idx() + 1)
 
             tooltip = vmmSerialConsole.can_connect(self.vm, dev)
             sensitive = not bool(tooltip)
@@ -946,7 +939,7 @@ class vmmConsolePages(vmmGObjectUI):
                 tooltip=tooltip, cb=self._console_menu_toggled, cbdata=dev)
 
     def _build_graphical_menu_items(self, menu_item_cb):
-        devs = self.vm.get_graphics_devices()
+        devs = self.vm.xmlobj.devices.graphics
         if len(devs) == 0:
             menu_item_cb(_("No graphical console available"),
                          radio=False, sensitive=False)
@@ -1016,7 +1009,7 @@ class vmmConsolePages(vmmGObjectUI):
         return self._viewer.console_get_pixbuf()
 
     def details_close_viewer(self):
-        return self._close_viewer()
+        return self._activate_unavailable_page(_("Viewer disconnected."))
 
     def details_activate_default_console_page(self):
         return self._activate_default_console_page()
